// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/12AISS8AO4/samples/sbtest/reg_mod.c $
// $Rev: 51900 $
// $Date: 2022-10-20 10:02:06 -0500 (Thu, 20 Oct 2022) $

// 12AISS8AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(void)
{
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ AISS8AO4_GSC_IBTR,	0x0000FFFF	},
		{ AISS8AO4_GSC_ICSR,	0x0000FFFF	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;
	u32			value;

	for (i = 0; (errs == 0) && (i < SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			ret			= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			ret			= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is low.
			ret	= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if (parm.value & bit)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			ret			= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is high.
			ret	= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. i %d, aiss8ao4_ioctl() failure, error %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	reg_mod_tests
*
*	Purpose:
*
*		Perform a test of the IOCTL service AISS8AO4_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_tests(int fd)
{
	int	errs	= 0;

	gsc_label("AISS8AO4_IOCTL_REG_MOD");
	errs	+= _service_test();
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


